/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.service;

import org.eclipse.jgit.api.errors.GitAPIException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.model.Commit;
import ru.ulstu.extractor.repository.CommitRepository;

import javax.transaction.Transactional;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CommitService {
    private final GitRepositoryService gitRepositoryService;
    private final CommitRepository commitRepository;
    private final AuthorService authorService;

    public CommitService(GitRepositoryService gitRepositoryService,
                         CommitRepository commitRepository,
                         AuthorService authorService) {
        this.gitRepositoryService = gitRepositoryService;
        this.commitRepository = commitRepository;
        this.authorService = authorService;
    }

    public Page<Commit> findPaginated(Pageable pageable, String repositoryUrl, String branchName) throws GitAPIException, IOException {
        int pageSize = pageable.getPageSize();
        int currentPage = pageable.getPageNumber();
        int startItem = currentPage * pageSize;
        List<Commit> commits = gitRepositoryService.getCommits(repositoryUrl, branchName);

        if (commits.size() < startItem) {
            commits = Collections.emptyList();
        } else {
            int toIndex = Math.min(startItem + pageSize, commits.size());
            commits = commits.subList(startItem, toIndex);
        }
        return new PageImpl<>(commits, PageRequest.of(currentPage, pageSize), commits.size());
    }

    public void delete(List<Commit> commitsToRemove) {
        commitRepository.deleteAll(commitsToRemove);
    }

    @Transactional
    public List<Commit> save(List<Commit> commits) {
        return commits.stream()
                .map(commit -> {
                    commit.setAuthor(authorService.findOrCreate(commit.getAuthor()));
                    return commitRepository.save(commit);
                }).collect(Collectors.toList());
    }
}


